@@ -10,8 +10,7 @@ import textwrap
 from argparse import Namespace
 from collections import defaultdict
 from itertools import chain
-from pathlib import Path
-from typing import Iterable, Mapping, Sequence, cast
+from typing import Collection, Iterable, Mapping, Sequence, cast
 
 import click
 import tomlkit
@@ -30,7 +29,12 @@ from pdm.cli.utils import (
     set_env_in_reg,
     translate_groups,
 )
-from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
+from pdm.exceptions import (
+    InvalidPyVersion,
+    NoPythonVersion,
+    PdmUsageError,
+    ProjectError,
+)
 from pdm.formats import FORMATS
 from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
 from pdm.models.caches import JSONFileCache
@@ -56,8 +60,22 @@ def do_lock(
     """Performs the locking process and update lockfile."""
     check_project_file(project)
     if refresh:
-        project.write_lockfile(project.lockfile)
-        return project.lockfile
+        locked_repo = project.locked_repository
+        repo = project.get_repository()
+        mapping: dict[str, Candidate] = {}
+        dependencies: dict[str, list[Requirement]] = {}
+        with project.core.ui.open_spinner("Re-calculating hashes..."):
+            for key, candidate in locked_repo.packages.items():
+                reqs, python_requires, summary = locked_repo.candidate_info[key]
+                candidate.hashes = repo.get_hashes(candidate)
+                candidate.summary = summary
+                candidate.requires_python = python_requires
+                ident = cast(str, key[0])
+                mapping[ident] = candidate
+                dependencies[ident] = list(map(parse_requirement, reqs))
+            lockfile = format_lockfile(project, mapping, dependencies)
+        project.write_lockfile(lockfile)
+        return mapping
     # TODO: multiple dependency definitions for the same package.
     provider = project.get_provider(strategy, tracked_names)
     if not requirements:
@@ -155,14 +173,14 @@ def check_lockfile(project: Project, raise_not_exist: bool = True) -> str | None
 def do_sync(
     project: Project,
     *,
-    groups: Sequence[str] = (),
+    groups: Collection[str] = (),
     dev: bool = True,
     default: bool = True,
     dry_run: bool = False,
     clean: bool = False,
     requirements: list[Requirement] | None = None,
-    tracked_names: Sequence[str] | None = None,
-    no_editable: bool = False,
+    tracked_names: Collection[str] | None = None,
+    no_editable: bool | Collection[str] = False,
     no_self: bool = False,
     reinstall: bool = False,
 ) -> None:
@@ -183,7 +201,7 @@ def do_sync(
         clean,
         dry_run,
         no_editable=no_editable,
-        install_self=not no_self and "default" in groups and bool(project.meta.name),
+        install_self=not no_self and "default" in groups and bool(project.name),
         use_install_cache=project.config["install.cache"],
         reinstall=reinstall,
     )
@@ -199,8 +217,8 @@ def do_add(
     sync: bool = True,
     save: str = "compatible",
     strategy: str = "reuse",
-    editables: Iterable[str] = (),
-    packages: Iterable[str] = (),
+    editables: Collection[str] = (),
+    packages: Collection[str] = (),
     unconstrained: bool = False,
     no_editable: bool = False,
     no_self: bool = False,
@@ -211,6 +229,8 @@ def do_add(
     check_project_file(project)
     if not editables and not packages:
         raise PdmUsageError("Must specify at least one package or editable package.")
+    if editables and no_editable:
+        raise PdmUsageError("Cannot use --no-editable with editable packages given.")
     if not group:
         group = "dev" if dev else "default"
     tracked_names: set[str] = set()
@@ -239,7 +259,9 @@ def do_add(
     deps_to_update = group_deps if unconstrained else requirements
     save_version_specifiers({group: deps_to_update}, resolved, save)
     if not dry_run:
-        project.add_dependencies(deps_to_update, group, dev)
+        project.add_dependencies(
+            deps_to_update, group, dev, replace_editable=no_editable
+        )
         project.write_lockfile(project.lockfile, False)
 
     if sync:
@@ -247,7 +269,7 @@ def do_add(
             project,
             groups=(group,),
             default=False,
-            no_editable=no_editable,
+            no_editable=no_editable and tracked_names,
             no_self=no_self,
             requirements=list(group_deps.values()),
             dry_run=dry_run,
@@ -265,7 +287,7 @@ def do_update(
     unconstrained: bool = False,
     top: bool = False,
     dry_run: bool = False,
-    packages: Sequence[str] = (),
+    packages: Collection[str] = (),
     sync: bool = True,
     no_editable: bool = False,
     no_self: bool = False,
@@ -356,7 +378,7 @@ def do_remove(
     dev: bool = False,
     group: str | None = None,
     sync: bool = True,
-    packages: Sequence[str] = (),
+    packages: Collection[str] = (),
     no_editable: bool = False,
     no_self: bool = False,
     dry_run: bool = False,
@@ -431,7 +453,12 @@ def do_list(
         if freeze:
             reqs = sorted(
                 (
-                    Requirement.from_dist(dist).as_line()
+                    Requirement.from_dist(dist)
+                    .as_line()
+                    .replace(
+                        "${PROJECT_ROOT}",
+                        project.root.absolute().as_posix().lstrip("/"),
+                    )
                     for dist in sorted(
                         working_set.values(), key=lambda d: d.metadata["Name"]
                     )
@@ -493,6 +520,7 @@ def do_init(
     project: Project,
     name: str = "",
     version: str = "",
+    description: str = "",
     license: str = "MIT",
     author: str = "",
     email: str = "",
@@ -503,16 +531,24 @@ def do_init(
         "project": {
             "name": name,
             "version": version,
-            "description": "",
+            "description": description,
             "authors": array_of_inline_tables([{"name": author, "email": email}]),
             "license": make_inline_table({"text": license}),
-            "urls": {"Homepage": ""},
             "dependencies": make_array([], True),
         },
-        "build-system": {"requires": ["pdm-pep517"], "build-backend": "pdm.pep517.api"},
+        "build-system": {
+            "requires": ["pdm-pep517>=0.12.0"],
+            "build-backend": "pdm.pep517.api",
+        },
     }
     if python_requires and python_requires != "*":
         data["project"]["requires-python"] = python_requires  # type: ignore
+    if name and version:
+        readme = next(project.root.glob("README*"), None)
+        if readme is None:
+            readme = project.root.joinpath("README.md")
+            readme.write_text(f"# {name}\n\n{description}\n")
+        data["project"]["readme"] = readme.name  # type: ignore
     get_specifier(python_requires)
     if not project.pyproject:
         project._pyproject = data
@@ -536,7 +572,7 @@ def do_use(
         python = python.strip()
 
     def version_matcher(py_version: PythonInfo) -> bool:
-        return project.python_requires.contains(str(py_version.version))
+        return project.python_requires.contains(str(py_version.version), True)
 
     if not project.cache_dir.exists():
         project.cache_dir.mkdir(parents=True)
@@ -546,14 +582,22 @@ def do_use(
     selected_python: PythonInfo | None = None
     if python and not ignore_remembered:
         if use_cache.has_key(python):
-            cached_python = PythonInfo.from_path(use_cache.get(python))
-            if version_matcher(cached_python):
+            path = use_cache.get(python)
+            cached_python = PythonInfo.from_path(path)
+            if not cached_python.valid:
+                project.core.ui.echo(
+                    f"The last selection is corrupted. {path!r}",
+                    fg="red",
+                    err=True,
+                )
+            elif version_matcher(cached_python):
                 project.core.ui.echo(
                     "Using the last selection, add '-i' to ignore it.",
                     fg="yellow",
                     err=True,
                 )
                 selected_python = cached_python
+
     if selected_python is None:
         found_interpreters = list(dict.fromkeys(project.find_interpreters(python)))
         matching_interperters = list(filter(version_matcher, found_interpreters))
@@ -567,24 +611,28 @@ def do_use(
                 "No python is found meeting the requirement "
                 f"{termui.green('python' + str(project.python_requires))}"
             )
-        if first or len(found_interpreters) == 1:
-            selected_python = found_interpreters[0]
+        if first or len(matching_interperters) == 1:
+            selected_python = matching_interperters[0]
         else:
             project.core.ui.echo("Please enter the Python interpreter to use")
-            for i, py_version in enumerate(found_interpreters):
+            for i, py_version in enumerate(matching_interperters):
                 project.core.ui.echo(
-                    f"{i}. {termui.green(py_version.executable)} "
+                    f"{i}. {termui.green(str(py_version.executable))} "
                     f"({py_version.identifier})"
                 )
             selection = click.prompt(
                 "Please select:",
-                type=click.Choice([str(i) for i in range(len(found_interpreters))]),
+                type=click.Choice([str(i) for i in range(len(matching_interperters))]),
                 default="0",
                 show_choices=False,
             )
-            selected_python = found_interpreters[int(selection)]
+            selected_python = matching_interperters[int(selection)]
         if python:
-            use_cache.set(python, selected_python.path)
+            use_cache.set(python, selected_python.path.as_posix())
+
+    if not selected_python.valid:
+        path = str(selected_python.executable)
+        raise InvalidPyVersion(f"Invalid Python interpreter: {path}")
 
     old_python = project.python if "python.path" in project.config else None
     project.core.ui.echo(
@@ -596,11 +644,11 @@ def do_use(
     project.python = selected_python
     if (
         old_python
-        and Path(old_python.path) != Path(selected_python.path)
+        and old_python.path != selected_python.path
         and not project.environment.is_global
     ):
         project.core.ui.echo(termui.cyan("Updating executable scripts..."))
-        project.environment.update_shebangs(selected_python.executable)
+        project.environment.update_shebangs(selected_python.executable.as_posix())
 
 
 def do_import(
