@@ -10,9 +10,8 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any, Iterable, Type, cast
 from urllib.parse import urlparse
 
-import atoml
-from pythonfinder import Finder
-from pythonfinder.environment import PYENV_INSTALLED, PYENV_ROOT
+import tomlkit
+from findpython import Finder
 
 from pdm import termui
 from pdm._types import Source
@@ -31,11 +30,13 @@ from pdm.utils import (
     atomic_open_for_write,
     cached_property,
     cd,
+    deprecation_warning,
     expand_env_vars_in_auth,
     find_project_root,
     find_python_in_path,
     get_in_project_venv_python,
-    is_venv_python,
+    get_venv_like_prefix,
+    normalize_name,
 )
 
 if TYPE_CHECKING:
@@ -46,16 +47,29 @@ if TYPE_CHECKING:
     from pdm.resolver.providers import BaseProvider
 
 
+PYENV_ROOT = os.path.expanduser(os.getenv("PYENV_ROOT", "~/.pyenv"))
+
+
 class Project:
-    """Core project class"""
+    """Core project class.
+
+    Args:
+        core: The core instance.
+        root_path: The root path of the project.
+        is_global: Whether the project is global.
+        global_config: The path to the global config file.
+    """
 
     PYPROJECT_FILENAME = "pyproject.toml"
     DEPENDENCIES_RE = re.compile(r"(?:(.+?)-)?dependencies")
     LOCKFILE_VERSION = "3.1"
-    GLOBAL_PROJECT = Path.home() / ".pdm" / "global-project"
 
     def __init__(
-        self, core: Core, root_path: str | Path | None, is_global: bool = False
+        self,
+        core: Core,
+        root_path: str | Path | None,
+        is_global: bool = False,
+        global_config: str | Path | None = None,
     ) -> None:
         self._pyproject: dict | None = None
         self._lockfile: dict | None = None
@@ -63,24 +77,34 @@ class Project:
         self._python: PythonInfo | None = None
         self.core = core
 
+        if global_config is None:
+            global_config = Path.home() / ".pdm/config.toml"
+        self.global_config = Config(Path(global_config), is_global=True)
+        global_project = Path(self.global_config["global_project.path"])
+
         if root_path is None:
             root_path = (
                 find_project_root(max_depth=self.global_config["project_max_depth"])
                 if not is_global
-                else self.GLOBAL_PROJECT
+                else global_project
             )
-        if not is_global and root_path is None and self.global_config["auto_global"]:
+        if (
+            not is_global
+            and root_path is None
+            and self.global_config["global_project.fallback"]
+        ):
             self.core.ui.echo(
                 "Project is not found, fallback to the global project",
                 fg="yellow",
                 err=True,
             )
-            root_path = self.GLOBAL_PROJECT
+            root_path = global_project
             is_global = True
 
         self.root = Path(root_path or "").absolute()
         self.is_global = is_global
         self.init_global_project()
+        self._lockfile_file = self.root / "pdm.lock"
 
     def __repr__(self) -> str:
         return f"<Project '{self.root.as_posix()}'>"
@@ -91,12 +115,17 @@ class Project:
 
     @property
     def lockfile_file(self) -> Path:
-        return self.root / "pdm.lock"
+        return self._lockfile_file
+
+    @lockfile_file.setter
+    def lockfile_file(self, path: str) -> None:
+        self._lockfile_file = Path(path).absolute()
+        self._lockfile = None
 
     @property
     def pyproject(self) -> dict | None:
         if not self._pyproject and self.pyproject_file.exists():
-            data = atoml.parse(self.pyproject_file.read_text("utf-8"))
+            data = tomlkit.parse(self.pyproject_file.read_text("utf-8"))
             self._pyproject = cast(dict, data)
         return self._pyproject
 
@@ -111,12 +140,16 @@ class Project:
             return {}
         return data.setdefault("tool", {}).setdefault("pdm", {})
 
+    @property
+    def name(self) -> str | None:
+        return self.meta.get("name")
+
     @property
     def lockfile(self) -> dict:
         if not self._lockfile:
             if not self.lockfile_file.is_file():
                 raise ProjectError("Lock file does not exist.")
-            data = atoml.parse(self.lockfile_file.read_text("utf-8"))
+            data = tomlkit.parse(self.lockfile_file.read_text("utf-8"))
             self._lockfile = cast(dict, data)
         return self._lockfile
 
@@ -133,12 +166,7 @@ class Project:
 
     @property
     def scripts(self) -> dict[str, str | dict[str, str]]:
-        return self.tool_settings.get("scripts")  # type: ignore
-
-    @cached_property
-    def global_config(self) -> Config:
-        """Read-and-writable configuration dict for global settings"""
-        return Config(self.GLOBAL_PROJECT.with_name("config.toml"), is_global=True)
+        return self.tool_settings.get("scripts", {})  # type: ignore
 
     @cached_property
     def project_config(self) -> Config:
@@ -149,6 +177,12 @@ class Project:
     def python(self) -> PythonInfo:
         if not self._python:
             self._python = self.resolve_interpreter()
+            if self._python.major < 3:
+                deprecation_warning(
+                    "Python 2.7 has reached EOL and PDM will remove the support "
+                    "in version 2.0. Please upgrade your Python to 3.6 or later.",
+                    raise_since="2.0",
+                )
         return self._python
 
     @python.setter
@@ -159,33 +193,39 @@ class Project:
     @property
     def python_executable(self) -> str:
         """For backward compatibility"""
-        return self.python.executable
+        return str(self.python.executable)
 
     def resolve_interpreter(self) -> PythonInfo:
         """Get the Python interpreter path."""
         config = self.config
-        if self.project_config.get("python.path") and not os.getenv(
-            "PDM_IGNORE_SAVED_PYTHON"
-        ):
-            saved_path = self.project_config["python.path"]
-            try:
-                return PythonInfo.from_path(saved_path)
-            except (ValueError, FileNotFoundError):
-                del self.project_config["python.path"]
+        if config.get("python.path") and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
+            saved_path = config["python.path"]
+            python = PythonInfo.from_path(saved_path)
+            if python.valid and self.python_requires.contains(
+                str(python.version), True
+            ):
+                return python
+            self.project_config.pop("python.path", None)
         if os.name == "nt":
             suffix = ".exe"
             scripts = "Scripts"
         else:
             suffix = ""
             scripts = "bin"
-        virtual_env = os.getenv("VIRTUAL_ENV")
-        if config["use_venv"] and virtual_env:
-            return PythonInfo.from_path(
+
+        # Resolve virtual environments from env-vars
+        virtual_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
+        if config["python.use_venv"] and virtual_env:
+            python = PythonInfo.from_path(
                 os.path.join(virtual_env, scripts, f"python{suffix}")
             )
+            if python.valid:
+                return python
 
         for py_version in self.find_interpreters():
-            if self.python_requires.contains(str(py_version.version)):
+            if py_version.valid and self.python_requires.contains(
+                str(py_version.version), True
+            ):
                 self.python = py_version
                 return py_version
 
@@ -201,9 +241,11 @@ class Project:
             env = GlobalEnvironment(self)
             # Rewrite global project's python requires to be
             # compatible with the exact version
-            env.python_requires = PySpecSet(f"=={self.python.version}")
+            env.python_requires = PySpecSet.equal_to(self.python.version)
             return env
-        if self.config["use_venv"] and is_venv_python(self.python.executable):
+        if self.config["python.use_venv"] and get_venv_like_prefix(
+            self.python.executable
+        ):
             # Only recognize venv created by python -m venv and virtualenv>20
             return GlobalEnvironment(self)
         return Environment(self)
@@ -307,6 +349,7 @@ class Project:
                 url=expand_env_vars_in_auth(s["url"]),
                 verify_ssl=s.get("verify_ssl", True),
                 name=s.get("name", urlparse(s["url"]).hostname),
+                type=s.get("type", "index"),
             )
             for s in sources
         ]
@@ -351,8 +394,11 @@ class Project:
         )
 
         repository = self.get_repository(cls=self.core.repository_class)
-        requires_python = self.environment.python_requires
         allow_prereleases = self.allow_prereleases
+        overrides = {
+            normalize_name(k): v
+            for k, v in self.tool_settings.get("overrides", {}).items()
+        }
         if strategy != "all" and not self.is_lockfile_compatible():
             self.core.ui.echo(
                 "Updating the whole lock file as it is not compatible with PDM",
@@ -361,11 +407,11 @@ class Project:
             )
             strategy = "all"
         if not for_install and strategy == "all":
-            return BaseProvider(repository, requires_python, allow_prereleases)
+            return BaseProvider(repository, allow_prereleases, overrides)
 
         locked_repository = self.locked_repository
         if for_install:
-            return BaseProvider(locked_repository, requires_python, allow_prereleases)
+            return BaseProvider(locked_repository, allow_prereleases, overrides)
         provider_class = (
             ReusePinProvider if strategy == "reuse" else EagerUpdateProvider
         )
@@ -373,8 +419,8 @@ class Project:
             locked_repository.all_candidates,
             tracked_names or (),
             repository,
-            requires_python,
             allow_prereleases,
+            overrides,
         )
 
     def get_reporter(
@@ -395,7 +441,7 @@ class Project:
         return SpinnerReporter(spinner or termui.DummySpinner(), requirements)
 
     def get_lock_metadata(self) -> dict[str, Any]:
-        content_hash = atoml.string("sha256:" + self.get_content_hash("sha256"))
+        content_hash = tomlkit.string("sha256:" + self.get_content_hash("sha256"))
         content_hash.trivia.trail = "\n\n"
         return {"lock_version": self.LOCKFILE_VERSION, "content_hash": content_hash}
 
@@ -406,7 +452,7 @@ class Project:
 
         if write:
             with atomic_open_for_write(self.lockfile_file) as fp:
-                atoml.dump(toml_data, fp)  # type: ignore
+                tomlkit.dump(toml_data, fp)  # type: ignore
             if show_message:
                 self.core.ui.echo(f"Changes are written to {termui.green('pdm.lock')}.")
             self._lockfile = None
@@ -416,9 +462,7 @@ class Project:
     def make_self_candidate(self, editable: bool = True) -> Candidate:
         req = parse_requirement(pip_shims.path_to_url(self.root.as_posix()), editable)
         req.name = self.meta.name
-        return Candidate(
-            req, self.environment, name=self.meta.name, version=self.meta.version
-        )
+        return Candidate(req, name=self.meta.name, version=self.meta.version)
 
     def get_content_hash(self, algo: str = "md5") -> str:
         # Only calculate sources and dependencies groups. Otherwise lock file is
@@ -429,6 +473,7 @@ class Project:
             "dev-dependencies": self.tool_settings.get("dev-dependencies", {}),
             "optional-dependencies": self.meta.get("optional-dependencies", {}),
             "requires-python": self.meta.get("requires-python", ""),
+            "overrides": self.tool_settings.get("overrides", {}),
         }
         pyproject_content = json.dumps(dump_data, sort_keys=True)
         hasher = hashlib.new(algo)
@@ -480,13 +525,15 @@ class Project:
         to_group: str = "default",
         dev: bool = False,
         show_message: bool = True,
+        replace_editable: bool = False,
     ) -> None:
         deps = self.get_pyproject_dependencies(to_group, dev).multiline(  # type: ignore
             True
         )
         for _, dep in requirements.items():
             matched_index = next(
-                (i for i, r in enumerate(deps) if dep.matches(r)), None
+                (i for i, r in enumerate(deps) if dep.matches(r, not replace_editable)),
+                None,
             )
             if matched_index is None:
                 deps.append(dep.as_line())
@@ -499,7 +546,7 @@ class Project:
         with atomic_open_for_write(
             self.pyproject_file.as_posix(), encoding="utf-8"
         ) as f:
-            atoml.dump(self.pyproject, f)  # type: ignore
+            tomlkit.dump(self.pyproject, f)  # type: ignore
         if show_message:
             self.core.ui.echo(
                 f"Changes are written to {termui.green('pyproject.toml')}."
@@ -509,9 +556,10 @@ class Project:
     @property
     def meta(self) -> Metadata:
         if not self.pyproject:
-            self.pyproject = {"project": atoml.table()}
-        m = Metadata(self.pyproject_file, self.pyproject.get("project", {}))
+            self.pyproject = {"project": tomlkit.table()}
+        m = Metadata(self.pyproject_file, False)
         m._metadata = self.pyproject.get("project", {})
+        m._tool_settings = self.tool_settings
         return m
 
     def init_global_project(self) -> None:
@@ -564,13 +612,15 @@ dependencies = ["pip", "setuptools", "wheel"]
         python: str | Path | None = None
 
         if not python_spec:
-            if config.get("python.use_pyenv", True) and PYENV_INSTALLED:
+            if config.get("python.use_pyenv", True) and os.path.exists(PYENV_ROOT):
                 pyenv_shim = os.path.join(PYENV_ROOT, "shims", "python3")
+                if os.name == "nt":
+                    pyenv_shim += ".bat"
                 if os.path.exists(pyenv_shim):
                     yield PythonInfo.from_path(pyenv_shim)
-                elif os.path.exists(pyenv_shim[:-1]):
-                    yield PythonInfo.from_path(pyenv_shim[:-1])
-            if config.get("use_venv"):
+                elif os.path.exists(pyenv_shim.replace("python3", "python")):
+                    yield PythonInfo.from_path(pyenv_shim.replace("python3", "python"))
+            if config.get("python.use_venv"):
                 python = get_in_project_venv_python(self.root)
                 if python:
                     yield PythonInfo.from_path(python)
@@ -590,9 +640,9 @@ dependencies = ["pip", "setuptools", "wheel"]
                         yield PythonInfo.from_path(python)
                 return
             args = [int(v) for v in python_spec.split(".") if v != ""]
-        finder = Finder()
-        for entry in finder.find_all_python_versions(*args):
-            yield PythonInfo.from_python_version(entry.py_version)
+        finder = Finder(resolve_symlinks=True)
+        for entry in finder.find_all(*args):
+            yield PythonInfo(entry)
         if not python_spec:
             this_python = getattr(sys, "_base_executable", sys.executable)
             yield PythonInfo.from_path(this_python)
