@@ -14,6 +14,7 @@ import sys
 import sysconfig
 import tempfile
 import urllib.parse as parse
+import warnings
 from contextlib import contextmanager
 from pathlib import Path
 from re import Match
@@ -26,10 +27,12 @@ from typing import (
     Iterator,
     TextIO,
     TypeVar,
+    cast,
     no_type_check,
     overload,
 )
 
+from packaging.version import Version
 from pip._vendor.packaging.tags import Tag
 from pip._vendor.requests import Session
 
@@ -161,6 +164,18 @@ def join_list_with(items: list[Any], sep: Any) -> list[Any]:
     return new_items[:-1]
 
 
+original_wheel_supported = PipWheel.supported
+original_support_index_min = PipWheel.support_index_min
+_has_find_most_preferred_tag = (
+    getattr(PipWheel, "find_most_preferred_tag", None) is not None
+)
+
+if _has_find_most_preferred_tag:
+    original_find: Any = PipWheel.find_most_preferred_tag
+else:
+    original_find = None
+
+
 @no_type_check
 @contextmanager
 def allow_all_wheels(enable: bool = True) -> Iterator:
@@ -171,8 +186,6 @@ def allow_all_wheels(enable: bool = True) -> Iterator:
     and set a new one, or else the results from the previous non-patched calls
     will interfere.
     """
-    from pdm.models.pip_shims import PipWheel
-
     if not enable:
         yield
         return
@@ -190,25 +203,14 @@ def allow_all_wheels(enable: bool = True) -> Iterator:
     ) -> int:
         return 0
 
-    has_find_most_preferred_tag = (
-        getattr(PipWheel, "find_most_preferred_tag", None) is not None
-    )
-
-    original_wheel_supported = PipWheel.supported
-    original_support_index_min = PipWheel.support_index_min
-    if has_find_most_preferred_tag:
-        original_find = PipWheel.find_most_preferred_tag
-    else:
-        original_find = None
-
     PipWheel.supported = _wheel_supported
     PipWheel.support_index_min = _wheel_support_index_min
-    if has_find_most_preferred_tag:
+    if _has_find_most_preferred_tag:
         PipWheel.find_most_preferred_tag = _find_most_preferred_tag
     yield
     PipWheel.supported = original_wheel_supported
     PipWheel.support_index_min = original_support_index_min
-    if has_find_most_preferred_tag:
+    if _has_find_most_preferred_tag:
         PipWheel.find_most_preferred_tag = original_find
 
 
@@ -254,13 +256,13 @@ def get_user_email_from_git() -> tuple[str, str]:
         return "", ""
     try:
         username = subprocess.check_output(
-            [git, "config", "user.name"], text=True
+            [git, "config", "user.name"], text=True, encoding="utf-8"
         ).strip()
     except subprocess.CalledProcessError:
         username = ""
     try:
         email = subprocess.check_output(
-            [git, "config", "user.email"], text=True
+            [git, "config", "user.email"], text=True, encoding="utf-8"
         ).strip()
     except subprocess.CalledProcessError:
         email = ""
@@ -330,16 +332,6 @@ def cd(path: str | Path) -> Iterator:
         os.chdir(_old_cwd)
 
 
-@contextmanager
-def temp_environ() -> Iterator:
-    environ = os.environ.copy()
-    try:
-        yield
-    finally:
-        os.environ.clear()
-        os.environ.update(environ)
-
-
 @contextmanager
 def open_file(url: str, session: Session | None = None) -> Iterator[BinaryIO]:
     if url.startswith("file://"):
@@ -375,7 +367,7 @@ def populate_link(
         candidate = finder.find_requirement(ireq, upgrade)
         if not candidate:
             return None
-        link = getattr(candidate, "link", candidate)
+        link = cast(Link, getattr(candidate, "link", candidate))
         ireq.link = link
     return ireq.link
 
@@ -440,10 +432,26 @@ def is_venv_python(interpreter: str | Path) -> bool:
     interpreter = Path(interpreter)
     if interpreter.parent.parent.joinpath("pyvenv.cfg").exists():
         return True
+
     virtual_env = os.getenv("VIRTUAL_ENV")
     return bool(virtual_env and is_path_relative_to(interpreter, virtual_env))
 
 
+def get_venv_like_prefix(interpreter: str | Path) -> Path | None:
+    """Check if the given interpreter path is from a virtualenv,
+    and return the prefix if found.
+    """
+    interpreter = Path(interpreter)
+    prefix = interpreter.parent.parent
+    if prefix.joinpath("pyvenv.cfg").exists():
+        return prefix
+
+    virtual_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
+    if virtual_env and is_path_relative_to(interpreter, virtual_env):
+        return Path(virtual_env)
+    return None
+
+
 def find_python_in_path(path: str | Path) -> Path | None:
     """Find a python interpreter from the given path, the input argument could be:
 
@@ -499,17 +507,53 @@ def is_editable(dist: Distribution) -> bool:
 
 def pdm_scheme(base: str) -> dict[str, str]:
     """Return a PEP 582 style install scheme"""
-    if "pdm" not in sysconfig._INSTALL_SCHEMES:  # type: ignore
+    if "pep582" not in sysconfig._INSTALL_SCHEMES:  # type: ignore
         bin_prefix = "Scripts" if os.name == "nt" else "bin"
-        sysconfig._INSTALL_SCHEMES["pdm"] = {  # type: ignore
-            "stdlib": "{base}/lib",
-            "platstdlib": "{base}/lib",
-            "purelib": "{base}/lib",
-            "platlib": "{base}/lib",
-            "include": "{base}/include",
-            "scripts": "{base}/%s" % bin_prefix,
-            "data": "{base}",
-            "prefix": "{base}",
-            "headers": "{base}/include",
+        sysconfig._INSTALL_SCHEMES["pep582"] = {  # type: ignore
+            "stdlib": "{pep582_base}/lib",
+            "platstdlib": "{pep582_base}/lib",
+            "purelib": "{pep582_base}/lib",
+            "platlib": "{pep582_base}/lib",
+            "include": "{pep582_base}/include",
+            "scripts": "{pep582_base}/%s" % bin_prefix,
+            "data": "{pep582_base}",
+            "prefix": "{pep582_base}",
+            "headers": "{pep582_base}/include",
         }
-    return sysconfig.get_paths("pdm", vars={"base": base}, expand=True)
+    return sysconfig.get_paths("pep582", vars={"pep582_base": base}, expand=True)
+
+
+def is_url(url: str) -> bool:
+    """Check if the given string is a URL"""
+    return bool(parse.urlparse(url).scheme)
+
+
+@functools.lru_cache()
+def fs_supports_symlink() -> bool:
+    if not hasattr(os, "symlink"):
+        return False
+    if sys.platform == "win32":
+        with tempfile.NamedTemporaryFile(prefix="TmP") as tmp_file:
+            temp_dir = os.path.dirname(tmp_file.name)
+            dest = os.path.join(temp_dir, "{}-{}".format(tmp_file.name, "b"))
+            try:
+                os.symlink(tmp_file.name, dest)
+                return True
+            except (OSError, NotImplementedError):
+                return False
+    else:
+        return True
+
+
+def deprecation_warning(
+    message: str, stacklevel: int = 1, raise_since: str | None = None
+) -> None:
+    """Show a deprecation warning with the given message and raise an error
+    after a specified version.
+    """
+    from pdm.__version__ import parsed_version
+
+    if raise_since is not None and parsed_version:
+        if parsed_version >= Version(raise_since):
+            raise DeprecationWarning(message)
+    warnings.warn(message, DeprecationWarning, stacklevel=stacklevel + 1)
